using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using DxVBLibA;

namespace Direct3DSample
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form
	{
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

    DirectX8Class DX8;
    Direct3D8 D3D;
    Direct3DDevice8 D3DDevice;

		public Form1()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

      DX8 = new DirectX8Class();
      D3D = DX8.Direct3DCreate();
      D3DDISPLAYMODE mode = new D3DDISPLAYMODE();
      D3D.GetAdapterDisplayMode((int)CONST_D3DCONST.D3DADAPTER_DEFAULT, ref mode);
      D3DPRESENT_PARAMETERS d3dpp = new D3DPRESENT_PARAMETERS();
      d3dpp.Windowed = 1;
      d3dpp.SwapEffect = CONST_D3DSWAPEFFECT.D3DSWAPEFFECT_COPY_VSYNC;
      d3dpp.BackBufferFormat = mode.Format;
      D3DDevice = D3D.CreateDevice((int)CONST_D3DCONST.D3DADAPTER_DEFAULT, CONST_D3DDEVTYPE.D3DDEVTYPE_HAL, this.Handle.ToInt32(), CONST_D3DCREATEFLAGS.D3DCREATE_SOFTWARE_VERTEXPROCESSING, ref d3dpp);
    }

    protected override void OnPaint(PaintEventArgs e) {
      Render();
    }

    struct CUSTOM_VERTEX {
      public float x;
      public float y;
      public float z;
      public float rhw;
      public uint color;
      public CUSTOM_VERTEX(float xx, float yy, float zz, float r, uint c) {
        x = xx;
        y = yy;
        z = zz;
        rhw = r;
        color = c;
      }
    }

    public void Render() {
      
      unsafe {
        CUSTOM_VERTEX[] vertices = new CUSTOM_VERTEX[]{ 
          new CUSTOM_VERTEX(10, 10, 0.5f, 1, 0xffff0000), 
          new CUSTOM_VERTEX(250, 250, 0.5f, 1, 0xff0ff000),
          new CUSTOM_VERTEX(10, 250, 0.5f, 1, 0xff0000ff),
          new CUSTOM_VERTEX(250, 10, 0.5f, 1, 0xff000000),
          new CUSTOM_VERTEX(250, 250, 0.5f, 1, 0xffffffff),
          new CUSTOM_VERTEX(10, 10, 0.5f, 1, 0xff888888)
        };
        int sizeOfVertices = sizeof(CUSTOM_VERTEX) * vertices.Length;

        Direct3DVertexBuffer8 vb;
        vb = D3DDevice.CreateVertexBuffer(sizeOfVertices, 0, (int)(CONST_D3DFVFFLAGS.D3DFVF_XYZRHW | CONST_D3DFVFFLAGS.D3DFVF_DIFFUSE), CONST_D3DPOOL.D3DPOOL_DEFAULT); 
        fixed (void* v = vertices) {
          int pData;
          byte* src = (byte*)v, dest;
          vb.Lock(0, sizeOfVertices, out pData, 0);
          dest = (byte*)pData;
          for (int i = 0; i < sizeOfVertices; i++)
            *dest++ = *src++;
        }

        D3DDevice.Clear(0, new IntPtr(0), CONST_D3DCLEARFLAGS.D3DCLEAR_TARGET, unchecked((int)0xff666699), 1.0f, 0);
        D3DDevice.BeginScene();
        D3DDevice.SetStreamSource(0, vb, sizeof(CUSTOM_VERTEX));
        D3DDevice.SetVertexShader((int)(CONST_D3DFVFFLAGS.D3DFVF_XYZRHW | CONST_D3DFVFFLAGS.D3DFVF_DIFFUSE));
        D3DDevice.DrawPrimitive(CONST_D3DPRIMITIVETYPE.D3DPT_TRIANGLELIST, 0, 2);
        D3DDevice.EndScene();
        D3DDevice.Present(new IntPtr(0), new IntPtr(0), 0, new IntPtr(0));
      }     
    }

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			this.Size = new System.Drawing.Size(300,300);
			this.Text = "Form1";
		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Form1());
		}
	}
}
